import {
  ModalForm, ProCard,
  ProForm, ProFormDependency, ProFormDigit, ProFormList, ProFormSelect,
  ProFormSwitch,
  ProFormText,
  ProFormUploadDragger,
  StepsForm
} from '@ant-design/pro-components';
import {FormattedMessage, useIntl} from '@umijs/max';
import React, {useEffect, useRef, useState} from 'react';
import {Button, Form, message, Modal, Switch, Tree} from 'antd';

import { Table, Transfer } from 'antd';
import type { ColumnsType, TableRowSelection } from 'antd/es/table/interface';
import type { TransferItem, TransferProps } from 'antd/es/transfer';
import {TransferDirection} from "antd/es/transfer";
import {
    postAlgorithmModelGetAlgorithmModelFkSelect,
    postAlgorithmModelGetAllVersionsById
} from "@/services/resource/AlgorithmModel";
import {
    postDeviceGetAllDeviceByGroup,
    postDeviceGetDeviceFkSelect,
    postDeviceGetDeviceList
} from "@/services/device/Device";
import {postDeviceGroupGetDeviceGroupTree} from "@/services/device/DeviceGroup";
import {putRoleUpdateDevicesInGroupModel} from "@/services/device/DeviceModelInGroup";
import {postProjectCreateProject} from "@/services/project/Project";
import {beforeUploadFile} from "@/utils/common";
import yaml from "js-yaml";
import {CloseOutlined, SnippetsOutlined} from "@ant-design/icons";
import {FormListActionType, ProFormDatePicker} from "@ant-design/pro-form/lib";
import {
    postModelVersionGetModelVersionById,
    postModelVersionGetModelVersionFkSelect
} from "@/services/resource/ModelVersion";

import difference from 'lodash/difference';
import TableTransferList from "@/pages/Setting/components/TableTransfer";
import {postProjectModelConfigGetProjectModelConfigById} from "@/services/project/ProjectModelConfig";


const formContainerWidth: number = 755;
const stepFormItemStyleProps: Record<string, any> = {
  width: formContainerWidth,
};
interface DataType extends API.Device{
    key: string,
}

interface ProjectModelConfigById {
  configId: number,
}


interface TableTransferProps extends TransferProps<TransferItem> {
    dataSource: DataType[];
    leftColumns: ColumnsType<DataType>;
    rightColumns: ColumnsType<DataType>;
}


export type UpdateFormProps = {
  updateModalOpen: boolean;
  handleModal: () => void;
  projectModelConfigId: number;
  groupId: number;
  reload: any;
};

const genFormFields = (type: number)=>{
  return (
    <>
      <ProFormText
        name="name"
        label="实验名称"
        width="md"
        tooltip="最长为 24 位，用于标定的唯一 id"
        placeholder="请输入名称"
        rules={[{ required: true }]}
      />
      <ProFormDatePicker name="date" label="日期" />
    </>
  )
}
const dataSource = [
  {
    key: '1',
    name: '胡彦斌',
    age: 32,
    address: '西湖区湖底公园1号',
  },
  {
    key: '2',
    name: '胡彦祖',
    age: 42,
    address: '西湖区湖底公园1号',
  },
];

const columns = [
  {
    title: '姓名',
    dataIndex: 'name',
    key: 'name',
  },
  {
    title: '年龄',
    dataIndex: 'age',
    key: 'age',
  },
  {
    title: '住址',
    dataIndex: 'address',
    key: 'address',
  },
];
const UpdateForm: React.FC<UpdateFormProps> = (props) => {

  const intl = useIntl();

  const [targetKeys, setTargetKeys] = useState<string[]>([]);
  const [current, setCurrent] = useState(0);

  const [configData, setConfigData] = useState([]);

  const [addrValue, setAddr] = useState('')
  const [projectModelConfig, setProjectModelConfig] = useState<API.ProjectModelConfig>({})

  const [deviceData, setDeviceData] = useState<DataType[]>([])
  const handleSelectChange = (value)=>{
    postModelVersionGetModelVersionById({ id: value }).then((resp) => {
      let data = resp.data.modelVersion.modelConfig?.params || [];
      setConfigData(data); // 更新状态
    });
  }

  useEffect(() => {
    if (addrValue) {
      postModelVersionGetModelVersionById({ id: 23 }).then((resp) => {
        let data = resp.data.modelVersion.modelConfig?.params || [];
        setConfigData(data); // 更新状态
      });
    }
  }, [addrValue]);

  const actionFormListRef = useRef<
    FormListActionType<{
      name: string;
    }>
  >();
  const getModelData = () => {
      // postDeviceGetDeviceFkSelect({keyword: ''}).then((res)=>{
      //     let result = (res.data?.list || []).map((v: any)=>{
      //         return {
      //             key: v.id,
      //             title: v.name,
      //             chosen: false,
      //         }
      //     })
      //     setModalData(result);
      // })
      // setMockData(tempMockData);
      // setTargetKeys(tempTargetKeys);

  };
  useEffect(() => {
      setTargetKeys([])
      setConfigData([])
      if (props.updateModalOpen) {
          // getModelData();
        console.log(props.projectModelConfigId)
          if (props.projectModelConfigId && props.projectModelConfigId !== 0) {
            postProjectModelConfigGetProjectModelConfigById({id: props.projectModelConfigId}).then((resp)=>{
              setProjectModelConfig(resp.data.ProjectModelConfig)
            })
          }
          // postDeviceGetDeviceList({groupFkId: 2, page: 1, pageSize:10}).then((resp)=>{
          //   // console.log(111, resp)
          // })
      } else {
      }
  }, [props.updateModalOpen])
  const handleChange = (newTargetKeys: string[]) => {
      setTargetKeys(newTargetKeys);
  };
  return (

    <StepsForm<{
      name: string;
    }>
      stepsProps={{
        labelPlacement: 'vertical',
      }}
      current={current}
      onCurrentChange={(value)=>{
          setCurrent(value)
          if (value == 1) {
              postDeviceGetAllDeviceByGroup({id: props.groupId}).then((resp)=>{
                  setDeviceData((resp.data?.list || []).map((v: API.Device)=>{
                      return {key: v.id, ...v}
                  }))
              })
          }
      }}
      onFinish={async () => {

        return true;
      }}
      formProps={{
        validateMessages: {
          required: '此项为必填项',
        },
      }}
      stepsFormRender={(dom, submitter) => {
        return (
          <Modal
            title="分步表单"
            width={1200}
            onCancel={() => {
              setCurrent(0);
              props.handleModal();
            }}
            open={props.updateModalOpen}
            footer={submitter}
            destroyOnClose
          >
            {dom}
          </Modal>
        );
      }}
    >
        <StepsForm.StepForm<{
            config: string;
        }>
            name="config"
            title="模型参数配置"
            stepProps={{
                description: '模型参数配置',
            }}
            onFocus={()=>{
                console.log("进入")
            }}
            onFinish={async (values: any) => {
                return true;
            }}
        >
            <>
                {
                    projectModelConfig.modelFkId && projectModelConfig.modelFkId !== 0 && (
                        <ProFormSelect
                            name="addr"
                            width="xs"
                            label="模型版本"
                            onChange={handleSelectChange}
                            initialValue={projectModelConfig.modelVersionFkId}
                            // dependencies 的内容会注入 request 中
                            request={async () => {
                                let resp =await postAlgorithmModelGetAllVersionsById({id: projectModelConfig.modelFkId})
                                return resp.data.list.map((v: any)=>{
                                    return {
                                        label: v.name,
                                        value: v.id
                                    }
                                })
                            }}
                        />
                    )
                }
                {
                    configData.length ?
                     (
                            configData.map((item:any, index)=> {
                                    if (typeof  item.default === 'number') {
                                        return (<ProFormDigit
                                            key={index}
                                            name={item.name}
                                            label={item.name}
                                            initialValue={item.default}
                                            tooltip={item.remark}
                                        />)
                                    } else if (typeof  item.default === 'boolean') {
                                        return (<ProFormSwitch
                                            key={index}
                                            name={item.name}
                                            label={item.name}
                                            initialValue={item.default}
                                            tooltip={item.remark}
                                        />)
                                    } else {
                                        return (<ProFormText
                                            key={index}
                                            name={item.name}
                                            label={item.name}
                                            initialValue={item.default}
                                            tooltip={item.remark}
                                        />)
                                    }
                                }
                            )
                        ): (projectModelConfig?.modelConfig || []).map((item, index)=>{
                            let value_type = typeof item.value
                            if (value_type === 'number') {
                                return (<ProFormDigit
                                    key={index}
                                    name={item.name}
                                    label={item.name}
                                    initialValue={item.value}
                                    tooltip={item.remark}
                                />)
                            } else if (value_type === 'boolean') {
                                return (<ProFormSwitch
                                    key={index}
                                    name={item.name}
                                    label={item.name}
                                    initialValue={item.value}
                                    tooltip={item.remark}
                                />)
                            } else if (value_type === 'string') {
                                return (<ProFormText
                                    key={index}
                                    name={item.name}
                                    label={item.name}
                                    initialValue={item.value}
                                    tooltip={item.remark}
                                />)
                            } else {
                                return
                            }
                        })
                }
            </>

        </StepsForm.StepForm>
          <StepsForm.StepForm<{
            config: string;
          }>
            name="config1"
            title="项目参数配置"
            stepProps={{
              description: '业务参数配置',
            }}
            onFinish={async (values: any) => {
              return true;
            }}
          >
            <>
                <ProForm.Group>
                    {
                        (projectModelConfig?.projectConfig || []).map((item, index)=>{
                            let value_type = typeof item.value
                            if (value_type === 'number') {
                                return (<ProFormDigit
                                    key={index}
                                    name={item.name}
                                    label={item.name}
                                    initialValue={item.value}
                                    tooltip={item.remark}
                                />)
                            } else if (value_type === 'boolean') {
                                return (<ProFormSwitch
                                    key={index}
                                    name={item.name}
                                    label={item.name}
                                    initialValue={item.value}
                                    tooltip={item.remark}
                                />)
                            } else if (value_type === 'string') {
                                return (<ProFormText
                                    key={index}
                                    name={item.name}
                                    label={item.name}
                                    initialValue={item.value}
                                    tooltip={item.remark}
                                />)
                            } else {
                                return
                            }
                        })
                    }
              {
                Object.keys(projectModelConfig?.projectConfig || {}).map((item:any, index)=> {
                    let value_type = typeof projectModelConfig.projectConfig[item]
                    console.log(value_type, projectModelConfig.projectConfig[item])

                    if (value_type === 'number') {
                      return (<ProFormDigit
                        key={index}
                        name={item}
                        label={item}
                        initialValue={projectModelConfig.projectConfig[item]}
                      />)
                    } else if (value_type === 'boolean') {
                      return (<ProFormSwitch
                          key={index}
                          name={item}
                          label={item}
                          initialValue={projectModelConfig.projectConfig[item]}
                      />)
                    } else if (value_type === 'string') {
                      return (<ProFormText
                          key={index}
                          width='md'
                          name={item}
                          label={item}
                          initialValue={projectModelConfig.projectConfig[item]}
                      />)
                    } else {
                        return
                    }
                  }
                )
              }
                </ProForm.Group>
            </>

          </StepsForm.StepForm>
      {/* 关联算法模型 */}

        <StepsForm.StepForm<{
            config: string;
        }>
            name="config3"
            title="设备配置"
            stepProps={{
                description: '设备关联配置',
            }}
            onFinish={async (values: any) => {
                console.log(targetKeys)
                return true;
            }}
        >
            <>
                {

                }
                <TableTransferList setTargetKeys={setTargetKeys} dataSource={deviceData}
                                   targetKeys={targetKeys}></TableTransferList>
            </>

        </StepsForm.StepForm>

    </StepsForm>



//     <ModalForm<API.DeviceModelInGroup>
//       title={intl.formatMessage({
//         id: 'common.modal.table.update.title',
//         defaultMessage: '$$$',
//       })}
//       open={props.updateModalOpen}
//       form={form}
//       autoFocusFirstInput
//       modalProps={{
//         destroyOnClose: true,
//         onCancel: () => props.handleModal(),
//       }}
//       submitTimeout={2000}
//       onFinish={async (values:any) => {
//           console.log(4444, props.values)
//           putRoleUpdateDevicesInGroupModel({groupId: props.values.groupFkId, modelId: props.values.modelFkId, deviceIds: values?.devices || []}).then(()=>{
//               props.handleModal();
//                 message.success(intl.formatMessage({id: 'common.success', defaultMessage: '$$$'}))
//           }).catch(()=>{
//                 message.error(intl.formatMessage({id: 'common.failure', defaultMessage: '$$$'}))
//           })
//         return true;
//       }}
//     >
//         <ProForm.Item name='devices'>
//             <Transfer
//                 dataSource={modalData}
//                 showSearch
//                 locale={{ itemUnit: '项', itemsUnit: '项', searchPlaceholder: '请输入搜索内容' }}
//                 targetKeys={targetKeys}
//                 onChange={handleChange}
//                 onSearch={handleSearch}
//                 style={{marginBottom: '20px'}}
//                 render={(item) => item.title}
//             />
//
//         </ProForm.Item>
// </ModalForm>

  )}
export default UpdateForm;
